*    Ŀ
*      FUNCTION NAME: SINCheck  (v 1.0)                             
*            AUTHORS: Mark Eddy, Bernie Bowers                      
*                     Hinton, Alberta  CANADA                       
*      COMPUSERVE ID: 72701,2352                                    
*       DATE WRITTEN: March 13, 1990                                
*       DISTRIBUTION: Public Domain                                 
*                                                                   
*      ABOUT THIS FUNCTION:                                         
*      The FUNCTION SINCheck validates a Canadian Social Insurance  
*      Number (SIN) passed to it as a nine digit character string.  
*      SINCheck returns a LOGICAL value.  This code does limited    
*      error checking.  Three criteria must be met for a return     
*      value of TRUE.  The SIN string must: be 9 characters in      
*      length; contain numeric characters only, and; satisfy a self 
*      checking code method.                                        
*                                                                   
*      THE VALIDATION PROCESS:                                      
*      Every SIN is an nine digit number, the last being a check    
*      digit assigned as a balancing number against the first       
*      eight.  This helps to reduce data entry errors, such as      
*      transposition.  The process can be reduced to three steps:   
*        1. subtotal the ODD positioned digits,                     
*        2. double the EVEN positioned digits and sum the digits    
*           of each result.  Total these sums, and                  
*        3. add the two subtotals for a result that will be evenly  
*           divisible by 10 if the SIN is valid.                    
*      There are other methods to validate a SIN, some involving    
*      lookup tables, but this seems to be best suited to the       
*      clipper syntax.  I welcome better methods / algorithms.      
*                                                                   
*      EXAMPLE:                                                     
*      to validate SIN "605903269"                                  
*        1. subtotal the ODD positioned digits (6+5+0+2+9 = 22),    
*        2. double the EVEN positioned digits (0,9,3,6 become       
*           00,18,06,12) and sum the digits of each result          
*           (0+0=0, 1+8=9, 0+6=6, 1+2=3).  Total these sums         
*           (0+9+6+3 = 18), and                                     
*        3. add the two subtotals for a result that will be evenly  
*           divisible by 10 if the SIN is valid (18+22 = 40).       
*    

* remove the *'s on the following code for a demonstration program
* CLEAR
* sinput = SPACE(11)
* DO WHILE LASTKEY() # 27
*   @ 24,00 SAY "Enter SIN "
*   @ 24,10 GET sinput PICT "999-999-999"
*   READ
*   @ 24,25 CLEAR
*   sin = SUBS(sinput,1,3) + SUBS(sinput,5,3) + SUBS(sinput,9,3)
*   @ 24,25 SAY sinput + " is " + IIF(SINCheck(sin), "Ok", "Invalid")
* ENDDO
* CLEAR
* @ 24,00 SAY "[Esc] pressed..."
* RETURN

FUNCTION SINCheck
 PARAMETERS _sin
 _sum = 0                                   && variable for balancing total
 _Ok = .F.                                  && variable for string validation
 IF LEN(_sin) = 09                          && check for proper length of SIN
   _Ok = .T.                                && string ok so far...
   FOR _count = 01 TO 09
     _char = SUBS(_sin,_count,01)           && pick a digit from SIN
     _num  = VAL(_char)                     && numeric value of digit
     _Ok = _Ok .AND. _char $ "1234567890"   && check each string character
     _sum = _sum + IIF(MOD(_count,02) # 0,; && determine postion (ODD|EVEN)
                     _num,;                 && calculate ODD position value
                     MOD(_num*02,10)+;      && calculate EVEN postion value
                     IIF(_num > 04,01,00))  &&   by summ of digits
   NEXT
 ENDIF
RETURN _Ok .AND. (MOD(_sum,10) = 00)        && string ok AND total balances
